<?php

/**
 * @file
 * 	Administrative forms and functions for the moon_phases module
 * @author Tom Camp <tom@campsitemedia.com>
 * @since 07/09/2010
 * @version 0.5
 */

/**
 * The form to set the Zip Code for your form /admin/settings/moon
 * @param array $form_state
 * @return array $form
 */
function moon_phases_admin_form( $form, &$form_state ) {

  $lat = variable_get( 'moon_lat', null );
  $lon = variable_get( 'moon_lon', null );

  if ( $lat && $lon ) {
    $form[ 'latlon' ] = array(
      '#markup' => t( 'Currently set to Latitude %lat, Longitude %lon', array( '%lat' => $lat, '%lon' => $lon ) )
    );
  }
  
  $form[ 'address' ] = array(
    '#type' => 'fieldset',
    '#title' => t( 'address' ),
    '#collapsible' => FALSE,
    '#tree' => TRUE,
  );
  
  $form[ 'address' ][ 'moon_city' ] = array(
    '#type' => 'textfield',
    '#title' => t( 'City' ),
    '#default_value' => variable_get( 'moon_city', NULL ),
    '#size' => 20,
    '#maxlength' => 64,
  );
  
  $form[ 'address' ][ 'moon_region' ] = array(
    '#type' => 'textfield',
    '#title' => t( 'Region' ),
    '#default_value' => variable_get( 'moon_region', NULL ),
    '#size' => 20,
    '#maxlength' => 64,
  );
  
  $countries = moon_phases_countries();
  $form[ 'address' ][ 'moon_country' ] = array(
    '#type' => 'select',
    '#title' => t( 'Country' ),
    '#default_value' => variable_get( 'moon_country', NULL ),
    '#options' => $countries,
  );

  $form[ 'address' ][ 'moon_zip' ] = array(
    '#type' => 'textfield',
    '#title' => t( 'Postal Code' ),
    '#default_value' => variable_get( 'moon_zip', null ),
    '#size' => 11,
    '#maxlength' => 10,
    '#description' => t( 'Format: 10001 or 10001-1234 or 10001 1234' ),
  );
  
  $form[ 'moon_geocoder' ] = array(
    '#type' => 'select',
    '#title' => t( 'Geo location service' ),
    '#default_value' => variable_get( 'moon_geocoder', 'google' ),
    '#options' => array(
      'google' => 'Google',
      'yahoo' => 'Yahoo',
      'yandex' => 'Yandex',
    ),
    '#description' => t( 'Choose the service which you would like you use to calculate your latitude and longitude.' ),
  );

  $form[ 'submit' ] = array(
    '#type' => 'submit',
    '#value' => t( 'Submit' )
  );

  return $form;

} // function moon_phases_admin_form()


/**
 * _validation function for moon_phases_admin_form
 *  Check length of zip code to make sure it is either 5 or 9 characters long
 *  Check to make sure that it only contains numerics, - or a space
 * @param array $form
 * @param array $form_state
 */
function moon_phases_admin_form_validate( $form, &$form_state ) {

  $zip = $form_state[ 'values' ][ 'address' ][ 'moon_zip' ];
  
  if ( $zip ) {
    if ( strlen( $zip ) < 5 || strlen( $zip ) > 10 || ( strlen( $zip ) < 10 && strlen( $zip ) > 5 )  ) {
      form_set_error( 'moon_zip', t( 'The zip code you entered is not valid.' ) );
      $valid = FALSE;
    }
    elseif ( is_numeric( $zip ) && strlen( $zip ) == 5 ) {
      $valid = TRUE;
    }
    elseif ( strlen( $zip ) == 10 ) {
      if ( preg_match( '/^[0-9]{5}([- ]?[0-9]{4})?$/', $form_state[ 'values' ][ 'moon_zip' ] ) ) {
        $valid = TRUE;
      }
      else {
        form_set_error( 'moon_zip', t( 'The zip code you entered is not valid.' ) );
        $valid = FALSE;
      }
    }
  }
  
  if ( empty ( $form_state[ 'values' ][ 'address' ][ 'moon_city' ] ) && empty ( $form_state[ 'values' ][ 'address' ][ 'moon_region' ] ) && empty ( $form_state[ 'values' ][ 'address' ][ 'moon_zip' ] ) ) {
    form_set_error( 'address', t( 'You must enter at least one address value to geocode.' ) );
  }
  else if ( ! $zip && ( empty ( $form_state[ 'values' ][ 'address' ][ 'moon_city' ] ) || empty ( $form_state[ 'values' ][ 'address' ][ 'moon_region' ] ) ) ) {
    if ( empty ( $form_state[ 'values' ][ 'address' ][ 'moon_city' ] ) ) {
      form_set_error( 'address][moon_city', t( 'City AND region are required if you do not include a postal code.' ) );
    }
    else {
      form_set_error( 'address][moon_region', t( 'City AND region are required if you do not include a postal code.' ) );
    }
    
  }

} // function moon_phases_admin_form_validate


/**
 * _submition function for moon_phases_admin_form
 * @param	array $form
 * @param	array $form_state
 */
function moon_phases_admin_form_submit( $form, &$form_state ) {

  foreach( $form_state[ 'values' ][ 'address' ] as $value ) {
    if ( $value != '' ) {
      $addy[] = $value;
    }
  }
  $address = implode( ', ', $addy );
  
  variable_set( 'moon_city', $form_state[ 'values' ][ 'address' ][ 'moon_city' ] );
  variable_set( 'moon_region', $form_state[ 'values' ][ 'address' ][ 'moon_region' ] );
  variable_set( 'moon_country', $form_state[ 'values' ][ 'address' ][ 'moon_country' ] );
  variable_set( 'moon_zip', $form_state[ 'values' ][ 'address' ][ 'moon_zip' ] );
  variable_set( 'moon_geocoder', $form_state[ 'values' ][ 'moon_geocoder' ] );
  
  moon_phases_admin_get_latlon( $address, $form_state[ 'values' ][ 'moon_geocoder' ] );

} // function moon_phases_admin_form_submit


/**
 * Get the latitude and longitude from the entered address
 * @param  string $address - comma separated address
 * @return boolean
 */
function moon_phases_admin_get_latlon( $address, $service ) {

  $geo_config = l( 'Geocoder configuration', 'admin/config/content/geocoder' );
  if ( $service == 'yahoo' ) {
    $geocoder_settings = variable_get( 'geocoder_settings', NULL );
    $key = ( isset( $geocoder_settings[ 'geocoder_apikey_yahoo' ] ) ) ? $geocoder_settings[ 'geocoder_apikey_yahoo' ] : NULL;
    
    if ( ! $key ) {
      drupal_set_message( t( 'You need to set the Yahoo Placefinder consumer key in the !link page.', array( '!link' => $geo_config ) ), 'error', FALSE );
      return FALSE;
    }
  }
  else if ( $service == 'yandex' ) {
    $geocoder_settings = variable_get( 'geocoder_settings', NULL );
    $key = ( isset( $geocoder_settings[ 'geocoder_apikey_yandex' ] ) ) ? $geocoder_settings[ 'geocoder_apikey_yandex' ] : NULL;
    
    if ( ! $key ) {
      drupal_set_message( t( 'You need to set the Yandex API Key in the !link page.', array( '!link' => $geo_config ) ), 'error', FALSE );
      return FALSE;
    }
  }
  $point = geocoder( $service, $address );
  
  $geoJSON = $point->out( 'json' );
  $latlon = json_decode( $geoJSON );

  if ( $point ) {
    variable_set( 'moon_lon', $latlon->coordinates[ 0 ] );
    variable_set( 'moon_lat', $latlon->coordinates[ 1 ] );
    
    drupal_set_message( t( ' Your location has been set to latitude %lat and longitude %lon.', array( '%lat' => $latlon->coordinates[ 1 ], '%lon' => $latlon->coordinates[ 0 ] ) ), 'status', FALSE );
  }
  else {
    drupal_set_message( t( 'Error geocoding your location.' ) );
  }

  return TRUE;

} // function moon_phases_admin_get_latlon


/**
 * 
 */
function moon_phases_countries() {
  
  return array(
    '' => '-------------------',
    'Afghanistan' => 'Afghanistan', 
    'Albania' => 'Albania', 
    'Algeria' => 'Algeria', 
    'American Samoa' => 'American Samoa', 
    'Andorra' => 'Andorra', 
    'Angola' => 'Angola', 
    'Anguilla' => 'Anguilla', 
    'Antarctica' => 'Antarctica', 
    'Antigua and Barbuda' => 'Antigua and Barbuda', 
    'Argentina' => 'Argentina', 
    'Armenia' => 'Armenia', 
    'Aruba' => 'Aruba', 
    'Australia' => 'Australia', 
    'Austria' => 'Austria', 
    'Azerbaijan' => 'Azerbaijan', 
    'Bahamas' => 'Bahamas', 
    'Bahrain' => 'Bahrain', 
    'Bangladesh' => 'Bangladesh', 
    'Barbados' => 'Barbados', 
    'Belarus' => 'Belarus', 
    'Belgium' => 'Belgium', 
    'Belize' => 'Belize', 
    'Benin' => 'Benin', 
    'Bermuda' => 'Bermuda', 
    'Bhutan' => 'Bhutan', 
    'Bolivia' => 'Bolivia', 
    'Bosnia and Herzegovina' => 'Bosnia and Herzegovina', 
    'Botswana' => 'Botswana', 
    'Bouvet Island' => 'Bouvet Island', 
    'Brazil' => 'Brazil', 
    'British Indian Ocean Territory' => 'British Indian Ocean Territory', 
    'Brunei Darussalam' => 'Brunei Darussalam', 
    'Bulgaria' => 'Bulgaria', 
    'Burkina Faso' => 'Burkina Faso', 
    'Burundi' => 'Burundi', 
    'Cambodia' => 'Cambodia', 
    'Cameroon' => 'Cameroon', 
    'Canada' => 'Canada', 
    'Cape Verde' => 'Cape Verde', 
    'Cayman Islands' => 'Cayman Islands', 
    'Central African Republic' => 'Central African Republic', 
    'Chad' => 'Chad', 
    'Chile' => 'Chile', 
    'China' => 'China', 
    'Christmas Island' => 'Christmas Island', 
    'Cocos (Keeling) Islands' => 'Cocos (Keeling) Islands', 
    'Colombia' => 'Colombia', 
    'Comoros' => 'Comoros', 
    'Congo' => 'Congo', 
    'Congo, The Democratic Republic of The' => 'Congo, The Democratic Republic of The', 
    'Cook Islands' => 'Cook Islands', 
    'Costa Rica' => 'Costa Rica', 
    'Cote D\'ivoire' => 'Cote D\'ivoire', 
    'Croatia' => 'Croatia', 
    'Cuba' => 'Cuba', 
    'Cyprus' => 'Cyprus', 
    'Czech Republic' => 'Czech Republic', 
    'Denmark' => 'Denmark', 
    'Djibouti' => 'Djibouti', 
    'Dominica' => 'Dominica', 
    'Dominican Republic' => 'Dominican Republic', 
    'Ecuador' => 'Ecuador', 
    'Egypt' => 'Egypt', 
    'El Salvador' => 'El Salvador', 
    'Equatorial Guinea' => 'Equatorial Guinea', 
    'Eritrea' => 'Eritrea', 
    'Estonia' => 'Estonia', 
    'Ethiopia' => 'Ethiopia', 
    'Falkland Islands (Malvinas)' => 'Falkland Islands (Malvinas)', 
    'Faroe Islands' => 'Faroe Islands', 
    'Fiji' => 'Fiji', 
    'Finland' => 'Finland', 
    'France' => 'France', 
    'French Guiana' => 'French Guiana', 
    'French Polynesia' => 'French Polynesia', 
    'French Southern Territories' => 'French Southern Territories', 
    'Gabon' => 'Gabon', 
    'Gambia' => 'Gambia', 
    'Georgia' => 'Georgia', 
    'Germany' => 'Germany', 
    'Ghana' => 'Ghana', 
    'Gibraltar' => 'Gibraltar', 
    'Greece' => 'Greece', 
    'Greenland' => 'Greenland', 
    'Grenada' => 'Grenada', 
    'Guadeloupe' => 'Guadeloupe', 
    'Guam' => 'Guam', 
    'Guatemala' => 'Guatemala', 
    'Guinea' => 'Guinea', 
    'Guinea-bissau' => 'Guinea-bissau', 
    'Guyana' => 'Guyana', 
    'Haiti' => 'Haiti', 
    'Heard Island and Mcdonald Islands' => 'Heard Island and Mcdonald Islands', 
    'Holy See (Vatican City State)' => 'Holy See (Vatican City State)', 
    'Honduras' => 'Honduras', 
    'Hong Kong' => 'Hong Kong', 
    'Hungary' => 'Hungary', 
    'Iceland' => 'Iceland', 
    'India' => 'India', 
    'Indonesia' => 'Indonesia', 
    'Iran, Islamic Republic of' => 'Iran, Islamic Republic of', 
    'Iraq' => 'Iraq', 
    'Ireland' => 'Ireland', 
    'Israel' => 'Israel', 
    'Italy' => 'Italy', 
    'Jamaica' => 'Jamaica', 
    'Japan' => 'Japan', 
    'Jordan' => 'Jordan', 
    'Kazakhstan' => 'Kazakhstan', 
    'Kenya' => 'Kenya', 
    'Kiribati' => 'Kiribati', 
    'Korea, Democratic People\'s Republic of' => 'Korea, Democratic People\'s Republic of', 
    'Korea, Republic of' => 'Korea, Republic of', 
    'Kuwait' => 'Kuwait', 
    'Kyrgyzstan' => 'Kyrgyzstan', 
    'Lao People\'s Democratic Republic' => 'Lao People\'s Democratic Republic', 
    'Latvia' => 'Latvia', 
    'Lebanon' => 'Lebanon', 
    'Lesotho' => 'Lesotho', 
    'Liberia' => 'Liberia', 
    'Libyan Arab Jamahiriya' => 'Libyan Arab Jamahiriya', 
    'Liechtenstein' => 'Liechtenstein', 
    'Lithuania' => 'Lithuania', 
    'Luxembourg' => 'Luxembourg', 
    'Macao' => 'Macao', 
    'Macedonia, The Former Yugoslav Republic of' => 'Macedonia, The Former Yugoslav Republic of', 
    'Madagascar' => 'Madagascar', 
    'Malawi' => 'Malawi', 
    'Malaysia' => 'Malaysia', 
    'Maldives' => 'Maldives', 
    'Mali' => 'Mali', 
    'Malta' => 'Malta', 
    'Marshall Islands' => 'Marshall Islands', 
    'Martinique' => 'Martinique', 
    'Mauritania' => 'Mauritania', 
    'Mauritius' => 'Mauritius', 
    'Mayotte' => 'Mayotte', 
    'Mexico' => 'Mexico', 
    'Micronesia, Federated States of' => 'Micronesia, Federated States of', 
    'Moldova, Republic of' => 'Moldova, Republic of', 
    'Monaco' => 'Monaco', 
    'Mongolia' => 'Mongolia', 
    'Montserrat' => 'Montserrat', 
    'Morocco' => 'Morocco', 
    'Mozambique' => 'Mozambique', 
    'Myanmar' => 'Myanmar', 
    'Namibia' => 'Namibia', 
    'Nauru' => 'Nauru', 
    'Nepal' => 'Nepal', 
    'Netherlands' => 'Netherlands', 
    'Netherlands Antilles' => 'Netherlands Antilles', 
    'New Caledonia' => 'New Caledonia', 
    'New Zealand' => 'New Zealand', 
    'Nicaragua' => 'Nicaragua', 
    'Niger' => 'Niger', 
    'Nigeria' => 'Nigeria', 
    'Niue' => 'Niue', 
    'Norfolk Island' => 'Norfolk Island', 
    'Northern Mariana Islands' => 'Northern Mariana Islands', 
    'Norway' => 'Norway', 
    'Oman' => 'Oman', 
    'Pakistan' => 'Pakistan', 
    'Palau' => 'Palau', 
    'Palestinian Territory, Occupied' => 'Palestinian Territory, Occupied', 
    'Panama' => 'Panama', 
    'Papua New Guinea' => 'Papua New Guinea', 
    'Paraguay' => 'Paraguay', 
    'Peru' => 'Peru', 
    'Philippines' => 'Philippines', 
    'Pitcairn' => 'Pitcairn', 
    'Poland' => 'Poland', 
    'Portugal' => 'Portugal', 
    'Puerto Rico' => 'Puerto Rico', 
    'Qatar' => 'Qatar', 
    'Reunion' => 'Reunion', 
    'Romania' => 'Romania', 
    'Russian Federation' => 'Russian Federation', 
    'Rwanda' => 'Rwanda', 
    'Saint Helena' => 'Saint Helena', 
    'Saint Kitts and Nevis' => 'Saint Kitts and Nevis', 
    'Saint Lucia' => 'Saint Lucia', 
    'Saint Pierre and Miquelon' => 'Saint Pierre and Miquelon', 
    'Saint Vincent and The Grenadines' => 'Saint Vincent and The Grenadines', 
    'Samoa' => 'Samoa', 
    'San Marino' => 'San Marino', 
    'Sao Tome and Principe' => 'Sao Tome and Principe', 
    'Saudi Arabia' => 'Saudi Arabia', 
    'Senegal' => 'Senegal', 
    'Serbia and Montenegro' => 'Serbia and Montenegro', 
    'Seychelles' => 'Seychelles', 
    'Sierra Leone' => 'Sierra Leone', 
    'Singapore' => 'Singapore', 
    'Slovakia' => 'Slovakia', 
    'Slovenia' => 'Slovenia', 
    'Solomon Islands' => 'Solomon Islands', 
    'Somalia' => 'Somalia', 
    'South Africa' => 'South Africa', 
    'South Georgia and The South Sandwich Islands' => 'South Georgia and The South Sandwich Islands', 
    'Spain' => 'Spain', 
    'Sri Lanka' => 'Sri Lanka', 
    'Sudan' => 'Sudan', 
    'Suriname' => 'Suriname', 
    'Svalbard and Jan Mayen' => 'Svalbard and Jan Mayen', 
    'Swaziland' => 'Swaziland', 
    'Sweden' => 'Sweden', 
    'Switzerland' => 'Switzerland', 
    'Syrian Arab Republic' => 'Syrian Arab Republic', 
    'Taiwan, Province of China' => 'Taiwan, Province of China', 
    'Tajikistan' => 'Tajikistan', 
    'Tanzania, United Republic of' => 'Tanzania, United Republic of', 
    'Thailand' => 'Thailand', 
    'Timor-leste' => 'Timor-leste', 
    'Togo' => 'Togo', 
    'Tokelau' => 'Tokelau', 
    'Tonga' => 'Tonga', 
    'Trinidad and Tobago' => 'Trinidad and Tobago', 
    'Tunisia' => 'Tunisia', 
    'Turkey' => 'Turkey', 
    'Turkmenistan' => 'Turkmenistan', 
    'Turks and Caicos Islands' => 'Turks and Caicos Islands', 
    'Tuvalu' => 'Tuvalu', 
    'Uganda' => 'Uganda', 
    'Ukraine' => 'Ukraine', 
    'United Arab Emirates' => 'United Arab Emirates', 
    'United Kingdom' => 'United Kingdom', 
    'United States' => 'United States', 
    'United States Minor Outlying Islands' => 'United States Minor Outlying Islands', 
    'Uruguay' => 'Uruguay', 
    'Uzbekistan' => 'Uzbekistan', 
    'Vanuatu' => 'Vanuatu', 
    'Venezuela' => 'Venezuela', 
    'Viet Nam' => 'Viet Nam', 
    'Virgin Islands, British' => 'Virgin Islands, British', 
    'Virgin Islands, U.S.' => 'Virgin Islands, U.S.', 
    'Wallis and Futuna' => 'Wallis and Futuna', 
    'Western Sahara' => 'Western Sahara', 
    'Yemen' => 'Yemen', 
    'Zambia' => 'Zambia', 
    'Zimbabwe' => 'Zimbabwe'
  );

} // function moon_phases_countries()